@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 3491 2008-01-22 17:46:51Z wwalker $"
-__version__   = "$Revision: 3491 $"
-__date__      = "$Date: 2008-01-22 12:46:51 -0500 (Tue, 22 Jan 2008) $"
+__id__        = "$Id: braillegenerator.py 3560 2008-02-11 17:35:26Z wwalker $"
+__version__   = "$Revision: 3560 $"
+__date__      = "$Date: 2008-02-11 12:35:26 -0500 (Mon, 11 Feb 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -331,9 +331,6 @@ class BrailleGenerator:
 
         text = ""
         state = obj.getState()
-        text = self._script.appendString(
-            text,
-            settings.brailleCheckBoxIndicators[indicatorindex])
 
         text = self._script.appendString(
             text, self._script.getDisplayedLabel(obj))
@@ -344,7 +341,9 @@ class BrailleGenerator:
             text, self._getTextForRole(obj, pyatspi.ROLE_CHECK_BOX))
 
         regions = []
-        componentRegion = braille.Component(obj, text)
+        componentRegion = braille.Component(
+            obj, text,
+            indicator=settings.brailleCheckBoxIndicators[indicatorindex])
         regions.append(componentRegion)
 
         return [regions, componentRegion]
@@ -372,9 +371,6 @@ class BrailleGenerator:
 
         text = ""
         state = obj.getState()
-        text = self._script.appendString(
-            text,
-            settings.brailleCheckBoxIndicators[indicatorindex])
 
         text = self._script.appendString(
             text, self._script.getDisplayedLabel(obj))
@@ -390,7 +386,9 @@ class BrailleGenerator:
                                       "")
 
         regions = []
-        componentRegion = braille.Component(obj, text)
+        componentRegion = braille.Component(
+            obj, text,
+            indicator=settings.brailleCheckBoxIndicators[indicatorindex])
         regions.append(componentRegion)
 
         return [regions, componentRegion]
@@ -764,9 +762,9 @@ class BrailleGenerator:
         #
         state = obj.getState()
         if state.contains(pyatspi.STATE_CHECKED):
-            text = self._script.appendString(
-                text,
-                settings.brailleCheckBoxIndicators[1])
+            indicator = settings.brailleCheckBoxIndicators[1]
+        else:
+            indicator = ''
 
         text = self._script.appendString(
             text, self._script.getDisplayedLabel(obj))
@@ -781,7 +779,7 @@ class BrailleGenerator:
                                       "")
 
         regions = []
-        componentRegion = braille.Component(obj, text)
+        componentRegion = braille.Component(obj, text, indicator=indicator)
         regions.append(componentRegion)
 
         return [regions, componentRegion]
@@ -968,10 +966,6 @@ class BrailleGenerator:
 
         text = ""
         state = obj.getState()
-        text = self._script.appendString(
-            settings.brailleRadioButtonIndicators[
-                int(state.contains(pyatspi.STATE_CHECKED))],
-            text)
 
         text = self._script.appendString(
             text, self._script.getDisplayedLabel(obj))
@@ -988,7 +982,10 @@ class BrailleGenerator:
         text = self._script.appendString(text, self._getTextForRole(obj))
 
         regions = []
-        componentRegion = braille.Component(obj, text)
+        componentRegion = braille.Component(
+            obj, text,
+            indicator=settings.brailleRadioButtonIndicators[ \
+            int(state.contains(pyatspi.STATE_CHECKED))])
         regions.append(componentRegion)
 
         return [regions, componentRegion]
@@ -1009,10 +1006,6 @@ class BrailleGenerator:
         text = ""
 
         state = obj.getState()
-        text = self._script.appendString(
-            settings.brailleRadioButtonIndicators[
-                state.contains(pyatspi.STATE_CHECKED)],
-            text)
 
         text = self._script.appendString(
             text, self._script.getDisplayedLabel(obj))
@@ -1028,7 +1021,10 @@ class BrailleGenerator:
                                       "")
 
         regions = []
-        componentRegion = braille.Component(obj, text)
+        componentRegion = braille.Component(
+            obj, text,
+            indicator=settings.brailleRadioButtonIndicators[ \
+            int(state.contains(pyatspi.STATE_CHECKED))])
         regions.append(componentRegion)
 
         return [regions, componentRegion]
