@@ -23,9 +23,9 @@ MouseButtonEvent, MouseMotionEvent, and SpeechEvent), and also provides a
 InputEventHandler class.  It is intended that instances of InputEventHandler
 will be used which should be used to handle all input events."""
 
-__id__        = "$Id: input_event.py 3275 2007-11-29 18:24:26Z richb $"
-__version__   = "$Revision: 3275 $"
-__date__      = "$Date: 2007-11-29 13:24:26 -0500 (Thu, 29 Nov 2007) $"
+__id__        = "$Id: input_event.py 3561 2008-02-11 18:11:45Z wwalker $"
+__version__   = "$Revision: 3561 $"
+__date__      = "$Date: 2008-02-11 13:11:45 -0500 (Mon, 11 Feb 2008) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -76,6 +76,16 @@ class KeyboardEvent(InputEvent):
             if value < 32:
                 event_string = chr(value + 0x40)
 
+        # Eliminate modifiers we're not interested in.
+        #
+        mask = (1 << settings.MODIFIER_ORCA |
+                1 << pyatspi.MODIFIER_ALT |
+                1 << pyatspi.MODIFIER_SHIFT |
+                1 << pyatspi.MODIFIER_CONTROL |
+                1 << pyatspi.MODIFIER_META2 |
+                1 << pyatspi.MODIFIER_META3)
+        event.modifiers = event.modifiers & mask
+
         InputEvent.__init__(self, KEYBOARD_EVENT)
         self.type = event.type
         self.hw_code = event.hw_code
