@@ -33,9 +33,9 @@ created in their own module.  The module defining the Script subclass
 is also required to have a 'getScript(app)' method that returns an
 instance of the Script subclass.  See default.py for an example."""
 
-__id__        = "$Id: script.py 2936 2007-09-27 19:46:45Z eitani $"
-__version__   = "$Revision: 2936 $"
-__date__      = "$Date: 2007-09-27 15:46:45 -0400 (Thu, 27 Sep 2007) $"
+__id__        = "$Id: script.py 3561 2008-02-11 18:11:45Z wwalker $"
+__version__   = "$Revision: 3561 $"
+__date__      = "$Date: 2008-02-11 13:11:45 -0500 (Mon, 11 Feb 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -189,6 +189,21 @@ class Script:
         """
         return where_am_I.WhereAmI(self)
     
+    def echoKey(self, keyEvent):
+        """Determine whether this script should echo the current key event.
+        Note that the keyEcho() method in orca.py will still take into 
+        account whatever the user's various preferences for key echoing
+        are, which may override what is return by this echoKey() method.
+
+        Arguments:
+        - keyEvent - the key event
+
+        Returns an indication of whether a key echo event should be
+        allowed to happen for this script.
+        """
+
+        return True
+
     def getBookmarks(self):
         """Returns the "bookmarks" class for this script.
         """
@@ -246,7 +261,14 @@ class Script:
         - appState: an object obtained from getAppState
         """
         return
-        
+
+    def getClickCount(self):
+        """Return the count of the number of clicks a user has made to one
+        of the keys on the keyboard.
+        """
+
+        return orca_state.clickCount
+
     # [[[WDW - There is a circular reference going on somewhere (see
     # bug 333168).  In the presence of this reference, the existence
     # of a __del__ method prevents the garbage collector from
@@ -304,6 +326,7 @@ class Script:
 
         Returns True if the event is of interest.
         """
+
         user_bindings = None
         user_bindings_map = settings.keyBindingsMap
         if user_bindings_map.has_key(self.__module__):
