@@ -19,9 +19,9 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 3328 2007-12-03 00:17:42Z joanied $"
-__version__   = "$Revision: 3328 $"
-__date__      = "$Date: 2007-12-02 19:17:42 -0500 (Sun, 02 Dec 2007) $"
+__id__        = "$Id: where_am_I.py 3561 2008-02-11 18:11:45Z wwalker $"
+__version__   = "$Revision: 3561 $"
+__date__      = "$Date: 2008-02-11 13:11:45 -0500 (Mon, 11 Feb 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -50,7 +50,7 @@ class WhereAmI:
         self._defaultButton = None
         self._lastAttributeString = ""
 
-    def whereAmI(self, obj, doubleClick, titleOrStatus):
+    def whereAmI(self, obj, basicOnly):
         """Speaks information about the current object of interest, including
         the object itself, which window it is in, which application, which
         workspace, etc.
@@ -70,107 +70,106 @@ class WhereAmI:
            \n  label=%s \
            \n  name=%s \
            \n  role=%s \
-           \n  mnemonics=%s \
+           \n  keybinding=%s \
            \n  parent label= %s \
            \n  parent name=%s \
            \n  parent role=%s \
-           \n  double-click=%s \
-           \n  titleOrStatus=%s" % \
+           \n  basicOnly=%s" % \
             (self._getObjLabel(obj),
              self._getObjName(obj),
              obj.getRoleName(),
-             self._script.getAcceleratorAndShortcut(obj),
+             self._script.getKeyBinding(obj),
              self._getObjLabel(obj.parent),
              self._getObjName(obj.parent),
              obj.parent.getRoleName(),
-             doubleClick,
-             titleOrStatus))
+             basicOnly))
 
         role = obj.getRole()
 
-        if titleOrStatus:
-            self._speakTitleOrStatus(obj, doubleClick)
+        toolbar = self._getToolbar(obj)
+        if toolbar:
+            self._speakToolbar(toolbar)
 
-        elif role == pyatspi.ROLE_CHECK_BOX:
-            self._speakCheckBox(obj, doubleClick)
+        if role == pyatspi.ROLE_CHECK_BOX:
+            self._speakCheckBox(obj, basicOnly)
 
         elif role == pyatspi.ROLE_RADIO_BUTTON:
-            self._speakRadioButton(obj, doubleClick)
+            self._speakRadioButton(obj, basicOnly)
 
         elif role == pyatspi.ROLE_COMBO_BOX:
-            self._speakComboBox(obj, doubleClick)
+            self._speakComboBox(obj, basicOnly)
 
         elif role == pyatspi.ROLE_SPIN_BUTTON:
-            self._speakSpinButton(obj, doubleClick)
+            self._speakSpinButton(obj, basicOnly)
 
         elif role == pyatspi.ROLE_PUSH_BUTTON:
-            self._speakPushButton(obj, doubleClick)
+            self._speakPushButton(obj, basicOnly)
 
         elif role == pyatspi.ROLE_SLIDER:
-            self._speakSlider(obj, doubleClick)
+            self._speakSlider(obj, basicOnly)
 
         elif role in [pyatspi.ROLE_MENU,
                       pyatspi.ROLE_MENU_ITEM,
                       pyatspi.ROLE_CHECK_MENU_ITEM,
                       pyatspi.ROLE_RADIO_MENU_ITEM]:
-            self._speakMenuItem(obj, doubleClick)
+            self._speakMenuItem(obj, basicOnly)
 
         elif role == pyatspi.ROLE_PAGE_TAB:
-            self._speakPageTab(obj, doubleClick)
+            self._speakPageTab(obj, basicOnly)
 
         elif role in [pyatspi.ROLE_TEXT,
                       pyatspi.ROLE_TERMINAL]:
-            self._speakText(obj, doubleClick)
+            self._speakText(obj, basicOnly)
 
         elif role == pyatspi.ROLE_TABLE_CELL:
-            self._speakTableCell(obj, doubleClick)
+            self._speakTableCell(obj, basicOnly)
 
         elif role == pyatspi.ROLE_LIST_ITEM:
-            self._speakListItem(obj, doubleClick)
+            self._speakListItem(obj, basicOnly)
 
         elif role == pyatspi.ROLE_PARAGRAPH:
-            self._speakParagraph(obj, doubleClick)
+            self._speakParagraph(obj, basicOnly)
 
         elif role == pyatspi.ROLE_ICON:
-            self._speakIconPanel(obj, doubleClick)
+            self._speakIconPanel(obj, basicOnly)
             
         elif role == pyatspi.ROLE_LINK:
-            self._speakLink(obj, doubleClick)
+            self._speakLink(obj, basicOnly)
 
         elif role == pyatspi.ROLE_TOGGLE_BUTTON:
-            self._speakToggleButton(obj, doubleClick)
+            self._speakToggleButton(obj, basicOnly)
 
         else:
-            self._speakGenericObject(obj, doubleClick)
+            self._speakGenericObject(obj, basicOnly)
 
-        if not (doubleClick or titleOrStatus):
+        if basicOnly:
             self._speakObjDescription(obj)
 
         return True
 
-    def _speakCheckBox(self, obj, doubleClick):
+    def _speakCheckBox(self, obj, basicOnly):
         """Checkboxes present the following information
         (an example is 'Enable speech, checkbox checked, Alt E'):
         1. label
         2. role
         3. state
-        4. mnemonic (i.e. Alt plus the underlined letter), if any
+        4. accelerator (i.e. Alt plus the underlined letter), if any
         """
 
         utterances = []
-        text = self._getObjLabelAndName(obj) + " " + \
+        text = self.getObjLabelAndName(obj) + " " + \
                rolenames.getSpeechForRoleName(obj)
         text = text + " " + self._getCheckBoxState(obj)
         utterances.append(text)
 
-        text = self._getObjMnemonic(obj)
-        utterances.append(text)
+        accelerator = self._getObjAccelerator(obj)
+        utterances.append(accelerator)
 
         debug.println(self._debugLevel, "check box utterances=%s" \
                       % utterances)
         speech.speakUtterances(utterances)
 
-    def _speakRadioButton(self, obj, doubleClick):
+    def _speakRadioButton(self, obj, basicOnly):
         """Radio Buttons present the following information (an example is
         'Punctuation Level, Some, Radio button, selected, item 2 of 4, Alt M'):
 
@@ -179,14 +178,14 @@ class WhereAmI:
         3. role
         4. state
         5. relative position
-        6. mnemonic (i.e. Alt plus the underlined letter), if any
+        6. accelerator (i.e. Alt plus the underlined letter), if any
         """
 
         utterances = []
         text = self._getGroupLabel(obj)
         utterances.append(text)
 
-        text = self._getObjLabelAndName(obj) + " " + \
+        text = self.getObjLabelAndName(obj) + " " + \
                rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
 
@@ -211,14 +210,14 @@ class WhereAmI:
         text = self._getPositionInGroup(obj)
         utterances.append(text)
 
-        text = self._getObjMnemonic(obj)
+        text = self._getObjAccelerator(obj)
         utterances.append(text)
 
         debug.println(self._debugLevel, "radio button utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakComboBox(self, obj, doubleClick):
+    def _speakComboBox(self, obj, basicOnly):
         """Comboboxes present the following information (an example is
         'Speech system: combo box, GNOME Speech Services, item 1 of 1,
         Alt S'):
@@ -226,7 +225,7 @@ class WhereAmI:
         2. role
         3. current value
         4. relative position
-        5. mnemonic (i.e. Alt plus the underlined letter), if any
+        5. accelerator (i.e. Alt plus the underlined letter), if any
         """
 
         utterances = []
@@ -243,21 +242,21 @@ class WhereAmI:
         text = self._getPositionInList(obj[0], name)
         utterances.append(text)
 
-        text = self._getObjMnemonic(obj)
-        utterances.append(text)
+        accelerator = self._getObjAccelerator(obj)
+        utterances.append(accelerator)
 
         debug.println(self._debugLevel, "combo box utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakSpinButton(self, obj, doubleClick):
+    def _speakSpinButton(self, obj, basicOnly):
         """Spin Buttons present the following information (an example is
         'Scale factor: spin button, 4.00, Alt F'):
 
         1. label
         2. role
         3. current value
-        4. mnemonic (i.e. Alt plus the underlined letter), if any
+        4. accelerator (i.e. Alt plus the underlined letter), if any
         """
 
         utterances = []
@@ -270,37 +269,37 @@ class WhereAmI:
         name = self._getObjName(obj)
         utterances.append(name)
         
-        text = self._getObjMnemonic(obj)
+        text = self._getObjAccelerator(obj)
         utterances.append(text)
 
         debug.println(self._debugLevel, "spin button utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakPushButton(self, obj, doubleClick):
+    def _speakPushButton(self, obj, basicOnly):
         """ Push Buttons present the following information (an example is
         'Apply button, Alt A'):
 
         1. label
         2. role
-        3. mnemonic (i.e. Alt plus the underlined letter), if any
+        3. accelerator (i.e. Alt plus the underlined letter), if any
         """
 
         utterances = []
-        text = self._getObjLabelAndName(obj)
+        text = self.getObjLabelAndName(obj)
         utterances.append(text)
 
         text = rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
 
-        text = self._getObjMnemonic(obj)
+        text = self._getObjAccelerator(obj)
         utterances.append(text)
 
         debug.println(self._debugLevel, "push button utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakSlider(self, obj, doubleClick):
+    def _speakSlider(self, obj, basicOnly):
         """Sliders present the following information (examples include
         'Pitch slider, 5.0, 56%'; 'Volume slider, 9.0, 100%'):
 
@@ -308,7 +307,7 @@ class WhereAmI:
         2. role
         3. value
         4. percentage (if possible)
-        5. mnemonic (i.e. Alt plus the underlined letter), if any
+        5. accelerator (i.e. Alt plus the underlined letter), if any
         """
 
         utterances = []
@@ -324,31 +323,31 @@ class WhereAmI:
         #
         utterances.append(_("%s percent") % values[1])
 
-        text = self._getObjMnemonic(obj)
+        text = self._getObjAccelerator(obj)
         utterances.append(text)
 
         debug.println(self._debugLevel, "slider utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakMenuItem(self, obj, doubleClick):
+    def _speakMenuItem(self, obj, basicOnly):
         """Menu items present the following information (examples include
         'File menu, Open..., Control + O, item 2 of 20, O', 'File menu,
         Wizards Menu, item 4 of 20, W'):
 
         1. Name of the menu containing the item, followed by its role
         2. item name, followed by its role (if a menu) followed by its
-        accelerator key, if any
+        accelerator key binding, if any
         3. relative position
-        4. mnemonic (i.e. Alt plus the underlined letter), if any
+        4. mnemonic (i.e. the underlined letter), if any
         """
 
         utterances = []
-        text = self._getObjLabelAndName(obj.parent) + " " + \
+        text = self.getObjLabelAndName(obj.parent) + " " + \
                rolenames.getSpeechForRoleName(obj.parent)
-        utterances.append(text)
+        utterances.append(text.strip())
 
-        text = self._getObjLabelAndName(obj)
+        text = self.getObjLabelAndName(obj)
         utterances.append(text)
 
         state = obj.getState()
@@ -385,36 +384,31 @@ class WhereAmI:
                 text = _("not selected")
             utterances.append(text)
 
-        text = self._getObjAccelerator(obj)
+        text = self._getObjAccelerator(obj, False, False)
         utterances.append(text)
 
         name = self._getObjName(obj)
         text = self._getPositionInList(obj.parent, name)
         utterances.append(text)
 
-        text = self._getObjShortcut(obj)
-
-        # The object's shortcut will be the full list of keys (e.g.
-        # Alt FO for the open menu item in the File menu). We only
-        # want to speak the shortcut associated with the menu item.
-        #
-        if text and obj.parent and obj.parent.getRole() == pyatspi.ROLE_MENU:
-            text = text[-1]
-        
-        utterances.append(text)
+        if obj.parent \
+           and obj.parent.getRole() in [pyatspi.ROLE_MENU, 
+                                        pyatspi.ROLE_MENU_BAR]:
+            text = self._getObjMnemonic(obj)
+            utterances.append(text)
 
         debug.println(self._debugLevel, "menu item utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakPageTab(self, obj, doubleClick):
+    def _speakPageTab(self, obj, basicOnly):
         """Tabs in a Tab List present the following information (an example
         is 'Tab list, braille page, item 2 of 5'):
 
         1. role
         2. label + 'page'
         3. relative position
-        4. mnemonic (i.e. Alt plus the underlined letter), if any
+        4. accelerator (i.e. Alt plus the underlined letter), if any
         """
 
         utterances = []
@@ -423,21 +417,21 @@ class WhereAmI:
 
         # Translators: "page" is the word for a page tab in a tab list.
         #
-        text = _("%s page") % self._getObjLabelAndName(obj)
+        text = _("%s page") % self.getObjLabelAndName(obj)
         utterances.append(text)
 
         name = self._getObjName(obj)
         text = self._getPositionInList(obj.parent, name)
         utterances.append(text)
 
-        text = self._getObjMnemonic(obj)
+        text = self._getObjAccelerator(obj)
         utterances.append(text)
 
         debug.println(self._debugLevel, "page utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakText(self, obj, doubleClick):
+    def _speakText(self, obj, basicOnly):
         """Text boxes present the following information (an example is
         'Source display: text, blank, Alt O'):
 
@@ -449,7 +443,7 @@ class WhereAmI:
             attibute information before  (bold "text")
             by 'selected' (single press)
             C. if the current line is blank/empty, 'blank'
-        4. mnemonic (i.e. Alt plus the underlined letter), if any
+        4. accelerator (i.e. Alt plus the underlined letter), if any
 
         Gaim, gedit, OpenOffice Writer and Terminal
         """
@@ -462,8 +456,8 @@ class WhereAmI:
         utterances.append(text)
 
         [textContents, startOffset, endOffset, selected] = \
-                       self._getTextContents(obj, doubleClick)
-        if doubleClick:
+                       self._getTextContents(obj, basicOnly)
+        if not basicOnly:
             # Speak character attributes.
             textContents = \
                 self._insertAttributes(obj, startOffset,
@@ -477,7 +471,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-        if doubleClick:
+        if not basicOnly:
             settings.verbalizePunctuationStyle = savedStyle
 
         utterances = []
@@ -490,14 +484,14 @@ class WhereAmI:
             text = Q_("text|selected")
             utterances.append(text)
 
-        text = self._getObjMnemonic(obj)
+        text = self._getObjAccelerator(obj)
         utterances.append(text)
 
         debug.println(self._debugLevel, "text utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakTableCell(self, obj, doubleClick):
+    def _speakTableCell(self, obj, basicOnly):
         """Tree Tables present the following information (an example is
         'Tree table, Mike Pedersen, item 8 of 10, tree level 2'):
 
@@ -527,7 +521,7 @@ class WhereAmI:
         speech.speakUtterances(utterances)
 
         utterances = []
-        if doubleClick:
+        if not basicOnly:
             table = parent.queryTable()
             row = table.getRowAtIndex(
               orca_state.locusOfFocus.getIndexInParent())
@@ -546,7 +540,7 @@ class WhereAmI:
         # Speak the remaining items.
         utterances = []
 
-        if not doubleClick:
+        if basicOnly:
             try:
                 table = parent.queryTable()
             except NotImplementedError:
@@ -589,7 +583,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakListItem(self, obj, doubleClick):
+    def _speakListItem(self, obj, basicOnly):
         """List items should be treated like tree cells:
 
         1. label, if any
@@ -603,7 +597,8 @@ class WhereAmI:
         utterances = []
 
         text = self._getObjLabel(obj)
-        utterances.append(text)
+        if text:
+            utterances.append(text)
 
         text = rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
@@ -649,13 +644,12 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakParagraph(self, obj, doubleClick):
+    def _speakParagraph(self, obj, basicOnly):
         """Speak a paragraph object.
         """
+        self._speakText(obj, basicOnly)
 
-        self._speakText(obj, doubleClick)
-
-    def _speakIconPanel(self, obj, doubleClick):
+    def _speakIconPanel(self, obj, basicOnly):
         """Speak the contents of the pane containing this icon. The
         number of icons in the pane is spoken. Plus the total number of
         selected icons. Plus the name of each of the selected icons.
@@ -669,7 +663,7 @@ class WhereAmI:
         childCount = panel.childCount
 
         utterances.append(_("Icon panel"))
-        utterances.append(self._getObjLabelAndName(obj))
+        utterances.append(self.getObjLabelAndName(obj))
 
         selectedItems = []
         totalSelectedItems = 0
@@ -699,19 +693,19 @@ class WhereAmI:
         itemString = _("on item %d of %d") % (currentItem, childCount)
         utterances.append(itemString)
 
-        if doubleClick:
+        if not basicOnly:
             for i in range(0, len(selectedItems)):
-                utterances.append(self._getObjLabelAndName(selectedItems[i]))
+                utterances.append(self.getObjLabelAndName(selectedItems[i]))
 
         speech.speakUtterances(utterances)
         
-    def _speakLink(self, obj, doubleClick):
+    def _speakLink(self, obj, basicOnly):
         """Speaks information about a link including protocol, domain 
         comparisons and size of file if possible
         
         Arguments:
         - obj: the icon object that currently has focus.
-        - doubleClick: was it a doubleclick event?
+        - basicOnly: True if the user is performing a standard/basic whereAmI.
         """
         
         # get the URI for the link of interest and parse it.
@@ -771,7 +765,7 @@ class WhereAmI:
 
         speech.speakUtterances([linkoutput, domainoutput, sizeoutput])
 
-    def _speakToggleButton(self, obj, doubleClick):
+    def _speakToggleButton(self, obj, basicOnly):
         """Speak toggle button information:
            1. Name/Label
            2. Role
@@ -779,7 +773,7 @@ class WhereAmI:
         """
 
         utterances = []
-        text = self._getObjLabelAndName(obj)
+        text = self.getObjLabelAndName(obj)
         utterances.append(text)
 
         text = rolenames.getSpeechForRoleName(obj)
@@ -831,13 +825,13 @@ class WhereAmI:
             #
             return _('%.2f megabytes') % (float(size) * .000001) 
 
-    def _speakGenericObject(self, obj, doubleClick):
+    def _speakGenericObject(self, obj, basicOnly):
         """Speak a generic object; one not specifically handled by
         other methods.
         """
 
         utterances = []
-        text = self._getObjLabelAndName(obj)
+        text = self.getObjLabelAndName(obj)
         utterances.append(text)
 
         text = rolenames.getSpeechForRoleName(obj)
@@ -873,7 +867,7 @@ class WhereAmI:
 
         return text
 
-    def _getObjLabelAndName(self, obj):
+    def getObjLabelAndName(self, obj):
         """Returns the object label plus the object name.
         """
 
@@ -912,13 +906,13 @@ class WhereAmI:
                 break
 
         if labelledBy:
-            text = self._getObjLabelAndName(labelledBy)
+            text = self.getObjLabelAndName(labelledBy)
         else:
             parent = obj.parent
             while parent and (parent.parent != parent):
                 if parent.getRole() in [pyatspi.ROLE_PANEL,
                                         pyatspi.ROLE_FILLER]:
-                    label = self._getObjLabelAndName(parent)
+                    label = self.getObjLabelAndName(parent)
                     if label and label != "":
                         text = label
                         break
@@ -995,44 +989,36 @@ class WhereAmI:
 
         return text
 
-    def _getObjMnemonic(self, obj):
-        """Returns the accellerator and/or shortcut for the object,
-        if either exists.
-        """
-
-        results = self._script.getAcceleratorAndShortcut(obj)
-
-        text = ""
-        if not results[1]:
-            text = results[0]
-        else:
-            text = results[0] + " " +  results[1]
-
-        return text
-
-    def _getObjAccelerator(self, obj):
+    def _getObjAccelerator(self,
+                           obj, 
+                           fallbackToMnemonic=True,
+                           fallbackToFullPath=True):
         """Returns the accelerator for the object, if it exists.
         """
 
-        results = self._script.getAcceleratorAndShortcut(obj)
+        # We'll try the real accelerator first, but fallback to the
+        # mnemonic if there is no accelerator.  This is done because
+        # some implementations, such as the Java platform, will
+        # only give us the mnemonic sometimes.
+        #
+        results = self._script.getKeyBinding(obj)
+        if results[2]:
+            return results[2]
 
-        text = ""
-        if results[0]:
-            text = results[0]
+        mnemonic = results[0]
+        if mnemonic and fallbackToMnemonic:
+            return mnemonic
 
-        return text
+        if fallbackToFullPath:
+            return results[1]
 
-    def _getObjShortcut(self, obj):
-        """Returns the shortcut for the object, if it exists.
+        return ""
+
+    def _getObjMnemonic(self, obj):
+        """Returns the mnemonic (a letter) for the object, if it exists.
         """
-
-        results = self._script.getAcceleratorAndShortcut(obj)
-
-        text = ""
-        if results[1]:
-            text = results[1]
-
-        return text
+        results = self._script.getKeyBinding(obj)
+        return results[0][-1:]
 
     def _getSliderValues(self, obj):
         """Returns the slider's current value and percentage.
@@ -1171,15 +1157,15 @@ class WhereAmI:
 
         return [textContents, startOffset, endOffset]
 
-    def _getTextSelections(self, obj, doubleClick):
+    def getTextSelections(self, obj, basicOnly):
         """Get all the text applicable text selections for the given object.
-        If the user doubleclicked, look to see if there are any previous 
-        or next text objects that also have selected text and add in their 
-        text contents.
+        If the user is doing a detailed whereAmI, look to see if there are
+        any previous or next text objects that also have selected text and
+        add in their text contents.
 
         Arguments:
         - obj: the text object to start extracting the selected text from.
-        - doubleClick: True if the user double-clicked the "where am I" key.
+        - basicOnly: True if the user is performing a standard/basic whereAmI.
 
         Returns: all the selected text contents plus the start and end
         offsets within the text for the given object.
@@ -1193,7 +1179,7 @@ class WhereAmI:
             [textContents, startOffset, endOffset] = \
                                             self._getTextSelection(obj)
 
-        if doubleClick:
+        if not basicOnly:
             current = obj
             morePossibleSelections = True
             while morePossibleSelections:
@@ -1315,7 +1301,7 @@ class WhereAmI:
 
         return [currentSelected, otherSelected]
 
-    def _getTextContents(self, obj, doubleClick):
+    def _getTextContents(self, obj, basicOnly):
         """Returns utterences for text.
 
         A. if no text on the current line is selected, the current line
@@ -1335,11 +1321,11 @@ class WhereAmI:
             (caretOffset, nSelections))
 
         [current, other] = self._hasTextSelections(obj)
-        if (doubleClick and (current or other)) or \
-           (not doubleClick and current):
+        if (not basicOnly and (current or other)) or \
+           (basicOnly and current):
             selected = True
             [textContents, startOffset, endOffset] = \
-                                  self._getTextSelections(obj, doubleClick)
+                                  self.getTextSelections(obj, basicOnly)
         else:
             # Get the line containing the caret
             #
@@ -1462,43 +1448,51 @@ class WhereAmI:
 
         return dictionary
 
-    def _speakTitleOrStatus(self, obj, doubleClick):
-        """When pressed a single time, Orca will speak the following
-        information:
+    def speakTitle(self, obj):
+        """Orca will speak the following information:
 
         1. The contents of the title bar of the application main window
         2. If in a dialog box within an application, the contents of the
         title bar of the dialog box.
         3. Orca will pause briefly between these two pieces of information
         so that the speech user can distinguish each.
-
-        If pressed twice, Orca will speak the status bar.
         """
 
         utterances = []
 
         results = self._getFrameAndDialog(obj)
 
-        if doubleClick:
-            if results[0]:
-                self._statusBar = None
-                self._getStatusBar(results[0])
-                if self._statusBar:
-                    self._speakStatusBar()
-            window = results[1] or results[0]
-            if window:
-                self._speakDefaultButton(window)
-        else:
-            if results[0]:
-                text = self._getObjLabelAndName(results[0])
-                utterances.append(text)
-            if results[1]:
-                text = self._getObjLabelAndName(results[1])
-                utterances.append(text)
+        if results[0]:
+            text = self.getObjLabelAndName(results[0])
+            utterances.append(text)
+        if results[1]:
+            text = self.getObjLabelAndName(results[1])
+            utterances.append(text)
 
-            debug.println(self._debugLevel, "titlebar utterances=%s" % \
-                          utterances)
-            speech.speakUtterances(utterances)
+        debug.println(self._debugLevel, "titlebar utterances=%s" % \
+                      utterances)
+        speech.speakUtterances(utterances)
+
+    def speakStatusBar(self, obj):
+        """Speak the contents of the status bar of the window with focus.
+        """
+ 
+        utterances = []
+ 
+        results = self._getFrameAndDialog(obj)
+
+        if results[0]:
+            self._statusBar = None
+            self._getStatusBar(results[0])
+            if self._statusBar:
+                self._speakStatusBar()
+        window = results[1] or results[0]
+        if window:
+            self._speakDefaultButton(window)
+
+        debug.println(self._debugLevel, "status bar utterances=%s" % \
+                      utterances)
+        speech.speakUtterances(utterances)
 
     def _getFrameAndDialog(self, obj):
         """Returns the frame and (possibly) the dialog containing
@@ -1511,7 +1505,7 @@ class WhereAmI:
         while parent and (parent.parent != parent):
             #debug.println(self._debugLevel,
             #              "_getFrameAndDialog: parent=%s, %s" % \
-            #              (parent.getRole(), self._getObjLabelAndName(parent)))
+            #              (parent.getRole(), self.getObjLabelAndName(parent)))
             if parent.getRole() == pyatspi.ROLE_FRAME:
                 results[0] = parent
             if parent.getRole() in [pyatspi.ROLE_DIALOG,
@@ -1528,7 +1522,7 @@ class WhereAmI:
             return
 
         # debug.println(self._debugLevel, "_findStatusBar: ROOT=%s, %s" % \
-        #               (obj.getRole(), self._getObjLabelAndName(obj)))
+        #               (obj.getRole(), self.getObjLabelAndName(obj)))
         state = obj.getState()
         managesDescendants = state.contains(pyatspi.STATE_MANAGES_DESCENDANTS)
         if managesDescendants:
@@ -1537,7 +1531,7 @@ class WhereAmI:
         for child in obj:
             # debug.println(self._debugLevel,
             #               "_findStatusBar: child=%s, %s" % \
-            #               (child.getRole(), self._getObjLabelAndName(child)))
+            #               (child.getRole(), self.getObjLabelAndName(child)))
             if child.getRole() == pyatspi.ROLE_STATUS_BAR:
                 self._statusBar = child
                 return
@@ -1577,7 +1571,7 @@ class WhereAmI:
         for child in obj:
             # debug.println(self._debugLevel,
             #               "_getDefaultButton: child=%s, %s" % \
-            #               (child.getRole(), self._getObjLabelAndName(child)))
+            #               (child.getRole(), self.getObjLabelAndName(child)))
             state = child.getState()
             if child.getRole() == pyatspi.ROLE_PUSH_BUTTON \
                 and state.contains(pyatspi.STATE_IS_DEFAULT):
@@ -1631,3 +1625,29 @@ class WhereAmI:
 
         if description and not description in [name, label]:
             speech.speak(description)
+
+    def _getToolbar(self, obj):
+        """Returns the toolbar containing the object or None if the object
+        is not contained in a toolbar.
+        """
+
+        parent = obj.parent
+        while parent and (parent.parent != parent):
+            if parent.getRole() == pyatspi.ROLE_TOOL_BAR:
+                return parent
+            parent = parent.parent
+
+        return None
+
+    def _speakToolbar(self, obj):
+        """Speaks the label and/or name of a toolbar, followed by its role.
+        """
+
+        utterances = []
+        text = self.getObjLabelAndName(obj) + " " + \
+               rolenames.getSpeechForRoleName(obj)
+        utterances.append(text.strip())
+
+        debug.println(self._debugLevel, "toolbar utterances=%s" \
+                      % utterances)
+        speech.speakUtterances(utterances)
